@@ -1,38 +1,58 @@
-//! ALB and API Gateway extension methods for `http::Request` types
+//! Extension methods for `http::Request` types
 
-use failure::Fail;
-use http::{header::CONTENT_TYPE, Request as HttpRequest};
+use crate::{request::RequestContext, Body};
+use lambda_runtime::Context;
+use query_map::QueryMap;
 use serde::{de::value::Error as SerdeError, Deserialize};
-use serde_json;
-use serde_urlencoded;
-
-use crate::{request::RequestContext, strmap::StrMap};
+use std::{error::Error, fmt};
 
 /// ALB/API gateway pre-parsed http query string parameters
-pub(crate) struct QueryStringParameters(pub(crate) StrMap);
+pub(crate) struct QueryStringParameters(pub(crate) QueryMap);
 
 /// API gateway pre-extracted url path parameters
 ///
 /// These will always be empty for ALB requests
-pub(crate) struct PathParameters(pub(crate) StrMap);
+pub(crate) struct PathParameters(pub(crate) QueryMap);
 
 /// API gateway configured
 /// [stage variables](https://docs.aws.amazon.com/apigateway/latest/developerguide/stage-variables.html)
 ///
 /// These will always be empty for ALB requests
-pub(crate) struct StageVariables(pub(crate) StrMap);
+pub(crate) struct StageVariables(pub(crate) QueryMap);
 
-/// Payload deserialization errors
-#[derive(Debug, Fail)]
+/// Request payload deserialization errors
+///
+/// Returned by [`RequestExt#payload()`](trait.RequestExt.html#tymethod.payload)
+#[derive(Debug)]
 pub enum PayloadError {
     /// Returned when `application/json` bodies fail to deserialize a payload
-    #[fail(display = "failed to parse payload from application/json")]
     Json(serde_json::Error),
     /// Returned when `application/x-www-form-urlencoded` bodies fail to deserialize a payload
-    #[fail(display = "failed to parse payload application/x-www-form-urlencoded")]
     WwwFormUrlEncoded(SerdeError),
 }
 
+impl fmt::Display for PayloadError {
+    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
+        match self {
+            PayloadError::Json(json) => writeln!(f, "failed to parse payload from application/json {}", json),
+            PayloadError::WwwFormUrlEncoded(form) => writeln!(
+                f,
+                "failed to parse payload from application/x-www-form-urlencoded {}",
+                form
+            ),
+        }
+    }
+}
+
+impl Error for PayloadError {
+    fn source(&self) -> Option<&(dyn Error + 'static)> {
+        match self {
+            PayloadError::Json(json) => Some(json),
+            PayloadError::WwwFormUrlEncoded(form) => Some(form),
+        }
+    }
+}
+
 /// Extentions for `lambda_http::Request` structs that
 /// provide access to [API gateway](https://docs.aws.amazon.com/apigateway/latest/developerguide/set-up-lambda-proxy-integrations.html#api-gateway-simple-proxy-for-lambda-input-format)
 /// and [ALB](https://docs.aws.amazon.com/elasticloadbalancing/latest/application/lambda-functions.html)
@@ -40,17 +60,16 @@ pub enum PayloadError {
 ///
 /// # Examples
 ///
-/// You can also access a request's body in deserialized format
-/// for payloads sent in `application/x-www-form-urlencoded` or
-/// `application/json` format.
+/// A request's body can be deserialized if its correctly encoded as per  
+/// the request's `Content-Type` header. The two supported content types are
+/// `application/x-www-form-urlencoded` and `application/json`.
 ///
 /// The following handler will work an http request body of `x=1&y=2`
 /// as well as `{"x":1, "y":2}` respectively.
 ///
 /// ```rust,no_run
-/// use lambda_runtime::{Context, error::HandlerError};
-/// use lambda_http::{lambda, Body, Request, Response, RequestExt};
-/// use serde_derive::Deserialize;
+/// use lambda_http::{service_fn, Error, Context, Body, IntoResponse, Request, Response, RequestExt};
+/// use serde::Deserialize;
 ///
 /// #[derive(Debug,Deserialize,Default)]
 /// struct Args {
@@ -60,14 +79,15 @@ pub enum PayloadError {
 ///   y: usize
 /// }
 ///
-/// fn main() {
-///   lambda!(handler)
+/// #[tokio::main]
+/// async fn main() -> Result<(), Error> {
+///   lambda_http::run(service_fn(add)).await?;
+///   Ok(())
 /// }
 ///
-/// fn handler(
-///   request: Request,
-///   ctx: Context
-/// ) -> Result<Response<Body>, HandlerError> {
+/// async fn add(
+///   request: Request
+/// ) -> Result<Response<Body>, Error> {
 ///   let args: Args = request.payload()
 ///     .unwrap_or_else(|_parse_err| None)
 ///     .unwrap_or_default();
@@ -93,39 +113,37 @@ pub trait RequestExt {
     /// name are expected, `query_string_parameters().get_all("many")` to retrieve them all.
     ///
     /// No query parameters
-    /// will yield an empty `StrMap`.
-    fn query_string_parameters(&self) -> StrMap;
+    /// will yield an empty `QueryMap`.
+    fn query_string_parameters(&self) -> QueryMap;
 
     /// Configures instance with query string parameters under #[cfg(test)] configurations
     ///
     /// This is intended for use in mock testing contexts.
-    #[cfg(test)]
     fn with_query_string_parameters<Q>(self, parameters: Q) -> Self
     where
-        Q: Into<StrMap>;
+        Q: Into<QueryMap>;
 
     /// Return pre-extracted path parameters, parameter provided in url placeholders
     /// `/foo/{bar}/baz/{boom}`,
     /// associated with the API gateway request. No path parameters
-    /// will yield an empty `StrMap`
+    /// will yield an empty `QueryMap`
     ///
     /// These will always be empty for ALB triggered requests
-    fn path_parameters(&self) -> StrMap;
+    fn path_parameters(&self) -> QueryMap;
 
     /// Configures instance with path parameters under #[cfg(test)] configurations
     ///
     /// This is intended for use in mock testing contexts.
-    #[cfg(test)]
     fn with_path_parameters<P>(self, parameters: P) -> Self
     where
-        P: Into<StrMap>;
+        P: Into<QueryMap>;
 
     /// Return [stage variables](https://docs.aws.amazon.com/apigateway/latest/developerguide/stage-variables.html)
     /// associated with the API gateway request. No stage parameters
-    /// will yield an empty `StrMap`
+    /// will yield an empty `QueryMap`
     ///
     /// These will always be empty for ALB triggered requests
-    fn stage_variables(&self) -> StrMap;
+    fn stage_variables(&self) -> QueryMap;
 
     /// Configures instance with stage variables under #[cfg(test)] configurations
     ///
@@ -133,7 +151,7 @@ pub trait RequestExt {
     #[cfg(test)]
     fn with_stage_variables<V>(self, variables: V) -> Self
     where
-        V: Into<StrMap>;
+        V: Into<QueryMap>;
 
     /// Return request context data assocaited with the ALB or API gateway request
     fn request_context(&self) -> RequestContext;
@@ -150,44 +168,48 @@ pub trait RequestExt {
     fn payload<D>(&self) -> Result<Option<D>, PayloadError>
     where
         for<'de> D: Deserialize<'de>;
+
+    /// Return the Lambda function context associated with the request
+    fn lambda_context(&self) -> Context;
+
+    /// Configures instance with lambda context
+    fn with_lambda_context(self, context: Context) -> Self;
 }
 
-impl RequestExt for HttpRequest<super::Body> {
-    fn query_string_parameters(&self) -> StrMap {
+impl RequestExt for http::Request<Body> {
+    fn query_string_parameters(&self) -> QueryMap {
         self.extensions()
             .get::<QueryStringParameters>()
             .map(|ext| ext.0.clone())
             .unwrap_or_default()
     }
 
-    #[cfg(test)]
     fn with_query_string_parameters<Q>(self, parameters: Q) -> Self
     where
-        Q: Into<StrMap>,
+        Q: Into<QueryMap>,
     {
         let mut s = self;
         s.extensions_mut().insert(QueryStringParameters(parameters.into()));
         s
     }
 
-    fn path_parameters(&self) -> StrMap {
+    fn path_parameters(&self) -> QueryMap {
         self.extensions()
             .get::<PathParameters>()
             .map(|ext| ext.0.clone())
             .unwrap_or_default()
     }
 
-    #[cfg(test)]
     fn with_path_parameters<P>(self, parameters: P) -> Self
     where
-        P: Into<StrMap>,
+        P: Into<QueryMap>,
     {
         let mut s = self;
         s.extensions_mut().insert(PathParameters(parameters.into()));
         s
     }
 
-    fn stage_variables(&self) -> StrMap {
+    fn stage_variables(&self) -> QueryMap {
         self.extensions()
             .get::<StageVariables>()
             .map(|ext| ext.0.clone())
@@ -197,7 +219,7 @@ impl RequestExt for HttpRequest<super::Body> {
     #[cfg(test)]
     fn with_stage_variables<V>(self, variables: V) -> Self
     where
-        V: Into<StrMap>,
+        V: Into<QueryMap>,
     {
         let mut s = self;
         s.extensions_mut().insert(StageVariables(variables.into()));
@@ -205,7 +227,23 @@ impl RequestExt for HttpRequest<super::Body> {
     }
 
     fn request_context(&self) -> RequestContext {
-        self.extensions().get::<RequestContext>().cloned().unwrap_or_default()
+        self.extensions()
+            .get::<RequestContext>()
+            .cloned()
+            .expect("Request did not contain a request context")
+    }
+
+    fn lambda_context(&self) -> Context {
+        self.extensions()
+            .get::<Context>()
+            .cloned()
+            .expect("Request did not contain a lambda context")
+    }
+
+    fn with_lambda_context(self, context: Context) -> Self {
+        let mut s = self;
+        s.extensions_mut().insert(context);
+        s
     }
 
     fn payload<D>(&self) -> Result<Option<D>, PayloadError>
@@ -213,14 +251,21 @@ impl RequestExt for HttpRequest<super::Body> {
         for<'de> D: Deserialize<'de>,
     {
         self.headers()
-            .get(CONTENT_TYPE)
+            .get(http::header::CONTENT_TYPE)
             .map(|ct| match ct.to_str() {
-                Ok("application/x-www-form-urlencoded") => serde_urlencoded::from_bytes::<D>(self.body().as_ref())
-                    .map_err(PayloadError::WwwFormUrlEncoded)
-                    .map(Some),
-                Ok("application/json") => serde_json::from_slice::<D>(self.body().as_ref())
-                    .map_err(PayloadError::Json)
-                    .map(Some),
+                Ok(content_type) => {
+                    if content_type.starts_with("application/x-www-form-urlencoded") {
+                        return serde_urlencoded::from_bytes::<D>(self.body().as_ref())
+                            .map_err(PayloadError::WwwFormUrlEncoded)
+                            .map(Some);
+                    } else if content_type.starts_with("application/json") {
+                        return serde_json::from_slice::<D>(self.body().as_ref())
+                            .map_err(PayloadError::Json)
+                            .map(Some);
+                    }
+
+                    Ok(None)
+                }
                 _ => Ok(None),
             })
             .unwrap_or_else(|| Ok(None))
@@ -229,11 +274,8 @@ impl RequestExt for HttpRequest<super::Body> {
 
 #[cfg(test)]
 mod tests {
-    use http::{HeaderMap, Request as HttpRequest};
-    use serde_derive::Deserialize;
-    use std::collections::HashMap;
-
-    use crate::{LambdaRequest, Request, RequestExt, StrMap};
+    use crate::{Body, Request, RequestExt};
+    use serde::Deserialize;
 
     #[test]
     fn requests_can_mock_query_string_parameters_ext() {
@@ -263,91 +305,100 @@ mod tests {
     }
 
     #[test]
-    fn requests_have_query_string_ext() {
-        let mut headers = HeaderMap::new();
-        headers.insert("Host", "www.rust-lang.org".parse().unwrap());
-        let mut query = HashMap::new();
-        query.insert("foo".to_owned(), vec!["bar".to_owned()]);
-        let lambda_request = LambdaRequest {
-            path: "/foo".into(),
-            headers,
-            query_string_parameters: StrMap(query.clone().into()),
-            ..LambdaRequest::default()
-        };
-        let actual = HttpRequest::from(lambda_request);
-        assert_eq!(actual.query_string_parameters(), StrMap(query.clone().into()));
-    }
-
-    #[test]
-    fn requests_have_form_post_parseable_payloads() {
-        let mut headers = HeaderMap::new();
-        headers.insert("Host", "www.rust-lang.org".parse().unwrap());
-        headers.insert("Content-Type", "application/x-www-form-urlencoded".parse().unwrap());
+    fn requests_have_form_post_parsable_payloads() {
         #[derive(Deserialize, PartialEq, Debug)]
         struct Payload {
             foo: String,
             baz: usize,
         }
-        let lambda_request = LambdaRequest {
-            path: "/foo".into(),
-            headers,
-            body: Some("foo=bar&baz=2".into()),
-            ..LambdaRequest::default()
-        };
-        let actual = HttpRequest::from(lambda_request);
-        let payload: Option<Payload> = actual.payload().unwrap_or_default();
+        let request = http::Request::builder()
+            .header("Content-Type", "application/x-www-form-urlencoded")
+            .body(Body::from("foo=bar&baz=2"))
+            .expect("failed to build request");
+        let payload: Option<Payload> = request.payload().unwrap_or_default();
         assert_eq!(
             payload,
             Some(Payload {
                 foo: "bar".into(),
                 baz: 2
             })
-        )
-    }
-
-    #[test]
-    fn requests_have_form_post_parseable_payloads_for_hashmaps() {
-        let mut headers = HeaderMap::new();
-        headers.insert("Host", "www.rust-lang.org".parse().unwrap());
-        headers.insert("Content-Type", "application/x-www-form-urlencoded".parse().unwrap());
-        let lambda_request = LambdaRequest {
-            path: "/foo".into(),
-            headers,
-            body: Some("foo=bar&baz=2".into()),
-            ..LambdaRequest::default()
-        };
-        let actual = HttpRequest::from(lambda_request);
-        let mut expected = HashMap::new();
-        expected.insert("foo".to_string(), "bar".to_string());
-        expected.insert("baz".to_string(), "2".to_string());
-        let payload: Option<HashMap<String, String>> = actual.payload().unwrap_or_default();
-        assert_eq!(payload, Some(expected))
+        );
     }
 
     #[test]
     fn requests_have_json_parseable_payloads() {
-        let mut headers = HeaderMap::new();
-        headers.insert("Host", "www.rust-lang.org".parse().unwrap());
-        headers.insert("Content-Type", "application/json".parse().unwrap());
         #[derive(Deserialize, PartialEq, Debug)]
         struct Payload {
             foo: String,
             baz: usize,
         }
-        let lambda_request: LambdaRequest<'_> = LambdaRequest {
-            path: "/foo".into(),
-            headers,
-            body: Some(r#"{"foo":"bar", "baz": 2}"#.into()),
-            ..LambdaRequest::default()
-        };
-        let actual = HttpRequest::from(lambda_request);
-        let payload: Option<Payload> = actual.payload().unwrap_or_default();
+        let request = http::Request::builder()
+            .header("Content-Type", "application/json")
+            .body(Body::from(r#"{"foo":"bar", "baz": 2}"#))
+            .expect("failed to build request");
+        let payload: Option<Payload> = request.payload().unwrap_or_default();
         assert_eq!(
             payload,
             Some(Payload {
                 foo: "bar".into(),
                 baz: 2
             })
-        )
+        );
+    }
+
+    #[test]
+    fn requests_match_form_post_content_type_with_charset() {
+        #[derive(Deserialize, PartialEq, Debug)]
+        struct Payload {
+            foo: String,
+            baz: usize,
+        }
+        let request = http::Request::builder()
+            .header("Content-Type", "application/x-www-form-urlencoded; charset=UTF-8")
+            .body(Body::from("foo=bar&baz=2"))
+            .expect("failed to build request");
+        let payload: Option<Payload> = request.payload().unwrap_or_default();
+        assert_eq!(
+            payload,
+            Some(Payload {
+                foo: "bar".into(),
+                baz: 2
+            })
+        );
+    }
+
+    #[test]
+    fn requests_match_json_content_type_with_charset() {
+        #[derive(Deserialize, PartialEq, Debug)]
+        struct Payload {
+            foo: String,
+            baz: usize,
+        }
+        let request = http::Request::builder()
+            .header("Content-Type", "application/json; charset=UTF-8")
+            .body(Body::from(r#"{"foo":"bar", "baz": 2}"#))
+            .expect("failed to build request");
+        let payload: Option<Payload> = request.payload().unwrap_or_default();
+        assert_eq!(
+            payload,
+            Some(Payload {
+                foo: "bar".into(),
+                baz: 2
+            })
+        );
+    }
+
+    #[test]
+    fn requests_omiting_content_types_do_not_support_parseable_payloads() {
+        #[derive(Deserialize, PartialEq, Debug)]
+        struct Payload {
+            foo: String,
+            baz: usize,
+        }
+        let request = http::Request::builder()
+            .body(Body::from(r#"{"foo":"bar", "baz": 2}"#))
+            .expect("failed to bulid request");
+        let payload: Option<Payload> = request.payload().unwrap_or_default();
+        assert_eq!(payload, None);
     }
 }
